//===============================================================================
// Microsoft patterns & practices
// CompositeUI Application Block Sample Visualizations
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// BUGFIX:
// * CAB Visualizer recognizes now if a WorkItem is disposed
// * CAB Visualizer is able to show an added hierarchy of workitems
// * See Post from Raymond: 
//     http://www.codeplex.com/smartclient/WorkItem/View.aspx?WorkItemId=7046
//===============================================================================

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.Practices.CompositeUI;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeUI.SmartParts;
using Microsoft.Practices.CompositeUI.Utility;

namespace Jbe.CABExtension.WinForms
{
	[System.ComponentModel.ToolboxItem(false)]
	public partial class WorkItemVisualization : UserControl
	{
		IVisualizer visualizer;

		public WorkItemVisualization()
		{
			InitializeComponent();
		}

		[InjectionConstructor]
		public WorkItemVisualization(IVisualizer visualizer, WorkItem workItem)
			: this()
		{
			this.visualizer = visualizer;

			workItemTree.Nodes.Clear();
            WorkItemAdded(this, new DataEventArgs<WorkItem>(visualizer.CabRootWorkItem));

			workItem.Workspaces["MainWorkspace"].Show(this, new SmartPartInfo("WorkItems", String.Empty));
		}

		private void AddTreeNode(TreeNodeCollection nodes, WorkItem wi)
		{
			bool firstChild = nodes.Count == 0;
			WorkItemTreeNode node = new WorkItemTreeNode(wi);
			nodes.Add(node);

			if (firstChild && node.Parent != null)
				node.Parent.Expand();
		}

		private WorkItemTreeNode GetTreeNode(WorkItem wi)
		{
			return GetTreeNode(wi, workItemTree.Nodes);
		}

		private WorkItemTreeNode GetTreeNode(WorkItem wi, TreeNodeCollection nodes)
		{
			foreach (TreeNode node in nodes)
			{
				WorkItemTreeNode wiNode = node as WorkItemTreeNode;

				if (wiNode != null && wiNode.WorkItem == wi)
					return wiNode;
			}

			foreach (TreeNode node in nodes)
			{
				WorkItemTreeNode wiNode = GetTreeNode(wi, node.Nodes);

				if (wiNode != null)
					return wiNode;
			}

			return null;
		}

        private void WorkItemAdded(object sender, DataEventArgs<WorkItem> e)
        {
            TreeNodeCollection collection = null;
            WorkItem wi = e.Data;

            if (wi.Parent == null) 
            { 
                collection = workItemTree.Nodes; 
            }
            else
            {
                WorkItemTreeNode node = GetTreeNode(wi.Parent);
                if (node != null) { collection = node.Nodes; }
            }

            if (collection != null)
            {
                if (wi.WorkItems != null)
                {
                    wi.WorkItems.Added += new EventHandler<DataEventArgs<WorkItem>>(WorkItemAdded);
                    wi.WorkItems.Removed += new EventHandler<DataEventArgs<WorkItem>>(WorkItemRemoved);
                }

                wi.IdChanged += new EventHandler<DataEventArgs<string>>(WorkItemIdChanged);
                wi.Activated += new EventHandler(WorkItemActivated);
                wi.Deactivated += new EventHandler(WorkItemDeactivated);
                wi.Disposed += new EventHandler(WorkItemDisposed);

                AddTreeNode(collection, wi);

                if (wi.WorkItems != null && wi.WorkItems.Count != 0)
                {
                    foreach (KeyValuePair<string, WorkItem> kvWorkItem in wi.WorkItems)
                    {
                        WorkItemAdded(this, new DataEventArgs<WorkItem>(kvWorkItem.Value));
                    }
                }
            }
        }

		private void WorkItemRemoved(object sender, DataEventArgs<WorkItem> e)
		{
            RemoveWorkItem(e.Data);
		}

		private void WorkItemActivated(object sender, EventArgs e)
		{
			Update((WorkItem)sender);
		}

		private void WorkItemDeactivated(object sender, EventArgs e)
		{
			Update((WorkItem)sender);
		}

		private void WorkItemIdChanged(object sender, DataEventArgs<string> e)
		{
			Update((WorkItem)sender);
		}

        private void WorkItemDisposed(object sender, EventArgs e)
        {
            RemoveWorkItem(sender as WorkItem);
        }

        private void RemoveWorkItem(WorkItem wi)
        {
            if (wi == null) { return; }

            wi.IdChanged -= new EventHandler<DataEventArgs<string>>(WorkItemIdChanged);
            wi.Activated -= new EventHandler(WorkItemActivated);
            wi.Deactivated -= new EventHandler(WorkItemDeactivated);
            wi.WorkItems.Added -= new EventHandler<DataEventArgs<WorkItem>>(WorkItemAdded);
            wi.WorkItems.Removed -= new EventHandler<DataEventArgs<WorkItem>>(WorkItemRemoved);
            wi.Disposed -= new EventHandler(WorkItemDisposed);

            WorkItemTreeNode node = GetTreeNode(wi);

            if (node != null && !node.TreeView.IsDisposed) { node.Remove(); }
        }

		private void Update(WorkItem wi)
		{
			WorkItemTreeNode node = GetTreeNode(wi);

			if (node != null)
				node.Update();
		}

        private void WorkItemTreeAfterSelect(object sender, TreeViewEventArgs e)
        {
            this.propertyGrid1.SelectedObject = ((WorkItemTreeNode)e.Node).WorkItem;
        }

		class WorkItemTreeNode : TreeNode
		{
			private WorkItem workItem;

			public WorkItem WorkItem
			{
				get { return workItem; }
			}

			public WorkItemTreeNode(WorkItem wi)
			{
				workItem = wi;
				Update();
			}

			public void Update()
			{
				this.Text = workItem.GetType().Name + " - " + workItem.ID;

				if (workItem.Status == WorkItemStatus.Active)
					BackColor = Color.Yellow;
				else
					BackColor = Color.White;
			}
		}
	}
}
